@@ -1,26 +1,19 @@
 from io import BytesIO
-import glob
-import os
-import numpy as np
+from pathlib import Path
+
 import pytest
 
 from matplotlib.testing.decorators import image_comparison
 from matplotlib import pyplot as plt
 import matplotlib.cm as cm
-import sys
-on_win = (sys.platform == 'win32')
 
 
-@image_comparison(baseline_images=['pngsuite'], extensions=['png'],
-                  tol=0.03)
+@image_comparison(['pngsuite.png'], tol=0.03)
 def test_pngsuite():
-    dirname = os.path.join(
-        os.path.dirname(__file__),
-        'baseline_images',
-        'pngsuite')
-    files = sorted(glob.iglob(os.path.join(dirname, 'basn*.png')))
+    files = sorted(
+        (Path(__file__).parent / "baseline_images/pngsuite").glob("basn*.png"))
 
-    fig = plt.figure(figsize=(len(files), 2))
+    plt.figure(figsize=(len(files), 2))
 
     for i, fname in enumerate(files):
         data = plt.imread(fname)
@@ -34,15 +27,6 @@ def test_pngsuite():
     plt.gca().set_xlim(0, len(files))
 
 
-def test_imread_png_uint16():
-    from matplotlib import _png
-    img = _png.read_png_int(os.path.join(os.path.dirname(__file__),
-                                     'baseline_images/test_png/uint16.png'))
-
-    assert (img.dtype == np.uint16)
-    assert np.sum(img.flatten()) == 134184960
-
-
 def test_truncated_file(tmpdir):
     d = tmpdir.mkdir('test')
     fname = str(d.join('test.png'))
