@@ -1,16 +1,23 @@
+import functools
+import itertools
+
 import pytest
 
 from mpl_toolkits.mplot3d import Axes3D, axes3d, proj3d, art3d
+import matplotlib as mpl
 from matplotlib import cm
-from matplotlib import path as mpath
+from matplotlib import colors as mcolors
 from matplotlib.testing.decorators import image_comparison, check_figures_equal
-from matplotlib.cbook.deprecation import MatplotlibDeprecationWarning
 from matplotlib.collections import LineCollection, PolyCollection
 from matplotlib.patches import Circle
 import matplotlib.pyplot as plt
 import numpy as np
 
 
+mpl3d_image_comparison = functools.partial(
+    image_comparison, remove_text=True, style='default')
+
+
 def test_aspect_equal_error():
     fig = plt.figure()
     ax = fig.add_subplot(111, projection='3d')
@@ -18,7 +25,7 @@ def test_aspect_equal_error():
         ax.set_aspect('equal')
 
 
-@image_comparison(baseline_images=['bar3d'], remove_text=True)
+@mpl3d_image_comparison(['bar3d.png'])
 def test_bar3d():
     fig = plt.figure()
     ax = fig.add_subplot(111, projection='3d')
@@ -30,17 +37,25 @@ def test_bar3d():
         ax.bar(xs, ys, zs=z, zdir='y', align='edge', color=cs, alpha=0.8)
 
 
-@image_comparison(
-    baseline_images=['bar3d_shaded'],
-    remove_text=True,
-    extensions=['png']
-)
+def test_bar3d_colors():
+    fig = plt.figure()
+    ax = fig.add_subplot(111, projection='3d')
+    for c in ['red', 'green', 'blue', 'yellow']:
+        xs = np.arange(len(c))
+        ys = np.zeros_like(xs)
+        zs = np.zeros_like(ys)
+        # Color names with same length as xs/ys/zs should not be split into
+        # individual letters.
+        ax.bar3d(xs, ys, zs, 1, 1, 1, color=c)
+
+
+@mpl3d_image_comparison(['bar3d_shaded.png'])
 def test_bar3d_shaded():
     x = np.arange(4)
     y = np.arange(5)
     x2d, y2d = np.meshgrid(x, y)
     x2d, y2d = x2d.ravel(), y2d.ravel()
-    z = x2d + y2d
+    z = x2d + y2d + 1  # Avoid triggering bug with zero-depth boxes.
 
     views = [(-60, 30), (30, 30), (30, -30), (120, -30)]
     fig = plt.figure(figsize=plt.figaspect(1 / len(views)))
@@ -54,11 +69,7 @@ def test_bar3d_shaded():
     fig.canvas.draw()
 
 
-@image_comparison(
-    baseline_images=['bar3d_notshaded'],
-    remove_text=True,
-    extensions=['png']
-)
+@mpl3d_image_comparison(['bar3d_notshaded.png'])
 def test_bar3d_notshaded():
     fig = plt.figure()
     ax = fig.add_subplot(111, projection='3d')
@@ -71,34 +82,60 @@ def test_bar3d_notshaded():
     fig.canvas.draw()
 
 
-@image_comparison(baseline_images=['contour3d'],
-                  remove_text=True, style='mpl20')
+def test_bar3d_lightsource():
+    fig = plt.figure()
+    ax = fig.add_subplot(1, 1, 1, projection="3d")
+
+    ls = mcolors.LightSource(azdeg=0, altdeg=90)
+
+    length, width = 3, 4
+    area = length * width
+
+    x, y = np.meshgrid(np.arange(length), np.arange(width))
+    x = x.ravel()
+    y = y.ravel()
+    dz = x + y
+
+    color = [cm.coolwarm(i/area) for i in range(area)]
+
+    collection = ax.bar3d(x=x, y=y, z=0,
+                          dx=1, dy=1, dz=dz,
+                          color=color, shade=True, lightsource=ls)
+
+    # Testing that the custom 90° lightsource produces different shading on
+    # the top facecolors compared to the default, and that those colors are
+    # precisely the colors from the colormap, due to the illumination parallel
+    # to the z-axis.
+    np.testing.assert_array_equal(color, collection._facecolors3d[1::6])
+
+
+@mpl3d_image_comparison(['contour3d.png'])
 def test_contour3d():
     fig = plt.figure()
     ax = fig.gca(projection='3d')
     X, Y, Z = axes3d.get_test_data(0.05)
-    cset = ax.contour(X, Y, Z, zdir='z', offset=-100, cmap=cm.coolwarm)
-    cset = ax.contour(X, Y, Z, zdir='x', offset=-40, cmap=cm.coolwarm)
-    cset = ax.contour(X, Y, Z, zdir='y', offset=40, cmap=cm.coolwarm)
+    ax.contour(X, Y, Z, zdir='z', offset=-100, cmap=cm.coolwarm)
+    ax.contour(X, Y, Z, zdir='x', offset=-40, cmap=cm.coolwarm)
+    ax.contour(X, Y, Z, zdir='y', offset=40, cmap=cm.coolwarm)
     ax.set_xlim(-40, 40)
     ax.set_ylim(-40, 40)
     ax.set_zlim(-100, 100)
 
 
-@image_comparison(baseline_images=['contourf3d'], remove_text=True)
+@mpl3d_image_comparison(['contourf3d.png'])
 def test_contourf3d():
     fig = plt.figure()
     ax = fig.gca(projection='3d')
     X, Y, Z = axes3d.get_test_data(0.05)
-    cset = ax.contourf(X, Y, Z, zdir='z', offset=-100, cmap=cm.coolwarm)
-    cset = ax.contourf(X, Y, Z, zdir='x', offset=-40, cmap=cm.coolwarm)
-    cset = ax.contourf(X, Y, Z, zdir='y', offset=40, cmap=cm.coolwarm)
+    ax.contourf(X, Y, Z, zdir='z', offset=-100, cmap=cm.coolwarm)
+    ax.contourf(X, Y, Z, zdir='x', offset=-40, cmap=cm.coolwarm)
+    ax.contourf(X, Y, Z, zdir='y', offset=40, cmap=cm.coolwarm)
     ax.set_xlim(-40, 40)
     ax.set_ylim(-40, 40)
     ax.set_zlim(-100, 100)
 
 
-@image_comparison(baseline_images=['contourf3d_fill'], remove_text=True)
+@mpl3d_image_comparison(['contourf3d_fill.png'])
 def test_contourf3d_fill():
     fig = plt.figure()
     ax = fig.gca(projection='3d')
@@ -107,14 +144,13 @@ def test_contourf3d_fill():
     # This produces holes in the z=0 surface that causes rendering errors if
     # the Poly3DCollection is not aware of path code information (issue #4784)
     Z[::5, ::5] = 0.1
-    cset = ax.contourf(X, Y, Z, offset=0, levels=[-0.1, 0], cmap=cm.coolwarm)
+    ax.contourf(X, Y, Z, offset=0, levels=[-0.1, 0], cmap=cm.coolwarm)
     ax.set_xlim(-2, 2)
     ax.set_ylim(-2, 2)
     ax.set_zlim(-1, 1)
 
 
-@image_comparison(baseline_images=['tricontour'], remove_text=True,
-                  style='mpl20', extensions=['png'])
+@mpl3d_image_comparison(['tricontour.png'])
 def test_tricontour():
     fig = plt.figure()
 
@@ -129,7 +165,7 @@ def test_tricontour():
     ax.tricontourf(x, y, z)
 
 
-@image_comparison(baseline_images=['lines3d'], remove_text=True)
+@mpl3d_image_comparison(['lines3d.png'])
 def test_lines3d():
     fig = plt.figure()
     ax = fig.gca(projection='3d')
@@ -141,11 +177,15 @@ def test_lines3d():
     ax.plot(x, y, z)
 
 
-# Reason for flakiness of SVG test is still unknown.
-@image_comparison(
-    baseline_images=['mixedsubplot'], remove_text=True,
-    extensions=['png', 'pdf',
-                pytest.param('svg', marks=pytest.mark.xfail(strict=False))])
+@check_figures_equal(extensions=["png"])
+def test_plot_scalar(fig_test, fig_ref):
+    ax1 = fig_test.gca(projection='3d')
+    ax1.plot([1], [1], "o")
+    ax2 = fig_ref.gca(projection='3d')
+    ax2.plot(1, 1, "o")
+
+
+@mpl3d_image_comparison(['mixedsubplot.png'])
 def test_mixedsubplots():
     def f(t):
         return np.cos(2*np.pi*t) * np.exp(-t)
@@ -155,8 +195,7 @@ def test_mixedsubplots():
 
     fig = plt.figure(figsize=plt.figaspect(2.))
     ax = fig.add_subplot(2, 1, 1)
-    l = ax.plot(t1, f(t1), 'bo',
-                t2, f(t2), 'k--', markerfacecolor='green')
+    ax.plot(t1, f(t1), 'bo', t2, f(t2), 'k--', markerfacecolor='green')
     ax.grid(True)
 
     ax = fig.add_subplot(2, 1, 2, projection='3d')
@@ -164,8 +203,8 @@ def test_mixedsubplots():
     R = np.hypot(X, Y)
     Z = np.sin(R)
 
-    surf = ax.plot_surface(X, Y, Z, rcount=40, ccount=40,
-                           linewidth=0, antialiased=False)
+    ax.plot_surface(X, Y, Z, rcount=40, ccount=40,
+                    linewidth=0, antialiased=False)
 
     ax.set_zlim3d(-1, 1)
 
@@ -183,18 +222,18 @@ def test_tight_layout_text(fig_test, fig_ref):
     ax2.text(.5, .5, .5, s='some string')
 
 
-@image_comparison(baseline_images=['scatter3d'], remove_text=True)
+@mpl3d_image_comparison(['scatter3d.png'])
 def test_scatter3d():
     fig = plt.figure()
     ax = fig.add_subplot(111, projection='3d')
     ax.scatter(np.arange(10), np.arange(10), np.arange(10),
                c='r', marker='o')
-    ax.scatter(np.arange(10, 20), np.arange(10, 20), np.arange(10, 20),
-               c='b', marker='^')
+    x = y = z = np.arange(10, 20)
+    ax.scatter(x, y, z, c='b', marker='^')
+    z[-1] = 0  # Check that scatter() copies the data.
 
 
-@image_comparison(baseline_images=['scatter3d_color'], remove_text=True,
-                  extensions=['png'])
+@mpl3d_image_comparison(['scatter3d_color.png'])
 def test_scatter3d_color():
     fig = plt.figure()
     ax = fig.add_subplot(111, projection='3d')
@@ -204,8 +243,108 @@ def test_scatter3d_color():
                color='b', marker='s')
 
 
-@image_comparison(baseline_images=['plot_3d_from_2d'], remove_text=True,
-                  extensions=['png'])
+@pytest.mark.parametrize('depthshade', [True, False])
+@check_figures_equal(extensions=['png'])
+def test_scatter3d_sorting(fig_ref, fig_test, depthshade):
+    """Test that marker properties are correctly sorted."""
+
+    y, x = np.mgrid[:10, :10]
+    z = np.arange(x.size).reshape(x.shape)
+
+    sizes = np.full(z.shape, 25)
+    sizes[0::2, 0::2] = 100
+    sizes[1::2, 1::2] = 100
+
+    facecolors = np.full(z.shape, 'C0')
+    facecolors[:5, :5] = 'C1'
+    facecolors[6:, :4] = 'C2'
+    facecolors[6:, 6:] = 'C3'
+
+    edgecolors = np.full(z.shape, 'C4')
+    edgecolors[1:5, 1:5] = 'C5'
+    edgecolors[5:9, 1:5] = 'C6'
+    edgecolors[5:9, 5:9] = 'C7'
+
+    linewidths = np.full(z.shape, 2)
+    linewidths[0::2, 0::2] = 5
+    linewidths[1::2, 1::2] = 5
+
+    x, y, z, sizes, facecolors, edgecolors, linewidths = [
+        a.flatten()
+        for a in [x, y, z, sizes, facecolors, edgecolors, linewidths]
+    ]
+
+    ax_ref = fig_ref.gca(projection='3d')
+    sets = (np.unique(a) for a in [sizes, facecolors, edgecolors, linewidths])
+    for s, fc, ec, lw in itertools.product(*sets):
+        subset = (
+            (sizes != s) |
+            (facecolors != fc) |
+            (edgecolors != ec) |
+            (linewidths != lw)
+        )
+        subset = np.ma.masked_array(z, subset, dtype=float)
+
+        # When depth shading is disabled, the colors are passed through as
+        # single-item lists; this triggers single path optimization. The
+        # following reshaping is a hack to disable that, since the optimization
+        # would not occur for the full scatter which has multiple colors.
+        fc = np.repeat(fc, sum(~subset.mask))
+
+        ax_ref.scatter(x, y, subset, s=s, fc=fc, ec=ec, lw=lw, alpha=1,
+                       depthshade=depthshade)
+
+    ax_test = fig_test.gca(projection='3d')
+    ax_test.scatter(x, y, z, s=sizes, fc=facecolors, ec=edgecolors,
+                    lw=linewidths, alpha=1, depthshade=depthshade)
+
+
+@pytest.mark.parametrize('azim', [-50, 130])  # yellow first, blue first
+@check_figures_equal(extensions=['png'])
+def test_marker_draw_order_data_reversed(fig_test, fig_ref, azim):
+    """
+    Test that the draw order does not depend on the data point order.
+
+    For the given viewing angle at azim=-50, the yellow marker should be in
+    front. For azim=130, the blue marker should be in front.
+    """
+    x = [-1, 1]
+    y = [1, -1]
+    z = [0, 0]
+    color = ['b', 'y']
+    ax = fig_test.add_subplot(projection='3d')
+    ax.scatter(x, y, z, s=3500, c=color)
+    ax.view_init(elev=0, azim=azim)
+    ax = fig_ref.add_subplot(projection='3d')
+    ax.scatter(x[::-1], y[::-1], z[::-1], s=3500, c=color[::-1])
+    ax.view_init(elev=0, azim=azim)
+
+
+@check_figures_equal(extensions=['png'])
+def test_marker_draw_order_view_rotated(fig_test, fig_ref):
+    """
+    Test that the draw order changes with the direction.
+
+    If we rotate *azim* by 180 degrees and exchange the colors, the plot
+    plot should look the same again.
+    """
+    azim = 130
+    x = [-1, 1]
+    y = [1, -1]
+    z = [0, 0]
+    color = ['b', 'y']
+    ax = fig_test.add_subplot(projection='3d')
+    # axis are not exactly invariant under 180 degree rotation -> deactivate
+    ax.set_axis_off()
+    ax.scatter(x, y, z, s=3500, c=color)
+    ax.view_init(elev=0, azim=azim)
+    ax = fig_ref.add_subplot(projection='3d')
+    ax.set_axis_off()
+    ax.scatter(x, y, z, s=3500, c=color[::-1])  # color reversed
+    ax.view_init(elev=0, azim=azim - 180)  # view rotated by 180 degrees
+
+
+@mpl3d_image_comparison(['plot_3d_from_2d.png'], tol=0.01)
 def test_plot_3d_from_2d():
     fig = plt.figure()
     ax = fig.add_subplot(111, projection='3d')
@@ -215,7 +354,7 @@ def test_plot_3d_from_2d():
     ax.plot(xs, ys, zs=0, zdir='y')
 
 
-@image_comparison(baseline_images=['surface3d'], remove_text=True)
+@mpl3d_image_comparison(['surface3d.png'])
 def test_surface3d():
     fig = plt.figure()
     ax = fig.gca(projection='3d')
@@ -230,8 +369,7 @@ def test_surface3d():
     fig.colorbar(surf, shrink=0.5, aspect=5)
 
 
-@image_comparison(baseline_images=['surface3d_shaded'], remove_text=True,
-                  extensions=['png'])
+@mpl3d_image_comparison(['surface3d_shaded.png'])
 def test_surface3d_shaded():
     fig = plt.figure()
     ax = fig.gca(projection='3d')
@@ -240,12 +378,12 @@ def test_surface3d_shaded():
     X, Y = np.meshgrid(X, Y)
     R = np.sqrt(X ** 2 + Y ** 2)
     Z = np.sin(R)
-    surf = ax.plot_surface(X, Y, Z, rstride=5, cstride=5,
-                           color=[0.25, 1, 0.25], lw=1, antialiased=False)
+    ax.plot_surface(X, Y, Z, rstride=5, cstride=5,
+                    color=[0.25, 1, 0.25], lw=1, antialiased=False)
     ax.set_zlim(-1.01, 1.01)
 
 
-@image_comparison(baseline_images=['text3d'])
+@mpl3d_image_comparison(['text3d.png'], remove_text=False)
 def test_text3d():
     fig = plt.figure()
     ax = fig.gca(projection='3d')
@@ -269,7 +407,7 @@ def test_text3d():
     ax.set_zlabel('Z axis')
 
 
-@image_comparison(baseline_images=['trisurf3d'], remove_text=True, tol=0.03)
+@mpl3d_image_comparison(['trisurf3d.png'], tol=0.03)
 def test_trisurf3d():
     n_angles = 36
     n_radii = 8
@@ -287,8 +425,7 @@ def test_trisurf3d():
     ax.plot_trisurf(x, y, z, cmap=cm.jet, linewidth=0.2)
 
 
-@image_comparison(baseline_images=['trisurf3d_shaded'], remove_text=True,
-                  tol=0.03, extensions=['png'])
+@mpl3d_image_comparison(['trisurf3d_shaded.png'], tol=0.03)
 def test_trisurf3d_shaded():
     n_angles = 36
     n_radii = 8
@@ -306,7 +443,7 @@ def test_trisurf3d_shaded():
     ax.plot_trisurf(x, y, z, color=[1, 0.5, 0], linewidth=0.2)
 
 
-@image_comparison(baseline_images=['wireframe3d'], remove_text=True)
+@mpl3d_image_comparison(['wireframe3d.png'])
 def test_wireframe3d():
     fig = plt.figure()
     ax = fig.add_subplot(111, projection='3d')
@@ -314,8 +451,7 @@ def test_wireframe3d():
     ax.plot_wireframe(X, Y, Z, rcount=13, ccount=13)
 
 
-@image_comparison(baseline_images=['wireframe3dzerocstride'], remove_text=True,
-                  extensions=['png'])
+@mpl3d_image_comparison(['wireframe3dzerocstride.png'])
 def test_wireframe3dzerocstride():
     fig = plt.figure()
     ax = fig.add_subplot(111, projection='3d')
@@ -323,8 +459,7 @@ def test_wireframe3dzerocstride():
     ax.plot_wireframe(X, Y, Z, rcount=13, ccount=0)
 
 
-@image_comparison(baseline_images=['wireframe3dzerorstride'], remove_text=True,
-                  extensions=['png'])
+@mpl3d_image_comparison(['wireframe3dzerorstride.png'])
 def test_wireframe3dzerorstride():
     fig = plt.figure()
     ax = fig.add_subplot(111, projection='3d')
@@ -350,37 +485,27 @@ def test_mixedsamplesraises():
         ax.plot_surface(X, Y, Z, cstride=50, rcount=10)
 
 
-@image_comparison(baseline_images=['quiver3d'], remove_text=True)
+@mpl3d_image_comparison(
+    ['quiver3d.png', 'quiver3d_pivot_middle.png', 'quiver3d_pivot_tail.png'])
 def test_quiver3d():
-    fig = plt.figure()
-    ax = fig.gca(projection='3d')
-
     x, y, z = np.ogrid[-1:0.8:10j, -1:0.8:10j, -1:0.6:3j]
-
     u = np.sin(np.pi * x) * np.cos(np.pi * y) * np.cos(np.pi * z)
     v = -np.cos(np.pi * x) * np.sin(np.pi * y) * np.cos(np.pi * z)
-    w = (np.sqrt(2.0 / 3.0) * np.cos(np.pi * x) * np.cos(np.pi * y) *
-            np.sin(np.pi * z))
+    w = (2/3)**0.5 * np.cos(np.pi * x) * np.cos(np.pi * y) * np.sin(np.pi * z)
+    for pivot in ['tip', 'middle', 'tail']:
+        ax = plt.figure().add_subplot(projection='3d')
+        ax.quiver(x, y, z, u, v, w, length=0.1, pivot=pivot, normalize=True)
 
+
+@check_figures_equal(extensions=["png"])
+def test_quiver3d_empty(fig_test, fig_ref):
+    fig_ref.add_subplot(projection='3d')
+    x = y = z = u = v = w = []
+    ax = fig_test.add_subplot(projection='3d')
     ax.quiver(x, y, z, u, v, w, length=0.1, pivot='tip', normalize=True)
 
 
-@image_comparison(baseline_images=['quiver3d_empty'], remove_text=True)
-def test_quiver3d_empty():
-    fig = plt.figure()
-    ax = fig.gca(projection='3d')
-
-    x, y, z = np.ogrid[-1:0.8:0j, -1:0.8:0j, -1:0.6:0j]
-
-    u = np.sin(np.pi * x) * np.cos(np.pi * y) * np.cos(np.pi * z)
-    v = -np.cos(np.pi * x) * np.sin(np.pi * y) * np.cos(np.pi * z)
-    w = (np.sqrt(2.0 / 3.0) * np.cos(np.pi * x) * np.cos(np.pi * y) *
-            np.sin(np.pi * z))
-
-    ax.quiver(x, y, z, u, v, w, length=0.1, pivot='tip', normalize=True)
-
-
-@image_comparison(baseline_images=['quiver3d_masked'], remove_text=True)
+@mpl3d_image_comparison(['quiver3d_masked.png'])
 def test_quiver3d_masked():
     fig = plt.figure()
     ax = fig.gca(projection='3d')
@@ -391,48 +516,14 @@ def test_quiver3d_masked():
 
     u = np.sin(np.pi * x) * np.cos(np.pi * y) * np.cos(np.pi * z)
     v = -np.cos(np.pi * x) * np.sin(np.pi * y) * np.cos(np.pi * z)
-    w = (np.sqrt(2.0 / 3.0) * np.cos(np.pi * x) * np.cos(np.pi * y) *
-            np.sin(np.pi * z))
+    w = (2/3)**0.5 * np.cos(np.pi * x) * np.cos(np.pi * y) * np.sin(np.pi * z)
     u = np.ma.masked_where((-0.4 < x) & (x < 0.1), u, copy=False)
     v = np.ma.masked_where((0.1 < y) & (y < 0.7), v, copy=False)
 
     ax.quiver(x, y, z, u, v, w, length=0.1, pivot='tip', normalize=True)
 
 
-@image_comparison(baseline_images=['quiver3d_pivot_middle'], remove_text=True,
-                  extensions=['png'])
-def test_quiver3d_pivot_middle():
-    fig = plt.figure()
-    ax = fig.gca(projection='3d')
-
-    x, y, z = np.ogrid[-1:0.8:10j, -1:0.8:10j, -1:0.6:3j]
-
-    u = np.sin(np.pi * x) * np.cos(np.pi * y) * np.cos(np.pi * z)
-    v = -np.cos(np.pi * x) * np.sin(np.pi * y) * np.cos(np.pi * z)
-    w = (np.sqrt(2.0 / 3.0) * np.cos(np.pi * x) * np.cos(np.pi * y) *
-            np.sin(np.pi * z))
-
-    ax.quiver(x, y, z, u, v, w, length=0.1, pivot='middle', normalize=True)
-
-
-@image_comparison(baseline_images=['quiver3d_pivot_tail'], remove_text=True,
-                  extensions=['png'])
-def test_quiver3d_pivot_tail():
-    fig = plt.figure()
-    ax = fig.gca(projection='3d')
-
-    x, y, z = np.ogrid[-1:0.8:10j, -1:0.8:10j, -1:0.6:3j]
-
-    u = np.sin(np.pi * x) * np.cos(np.pi * y) * np.cos(np.pi * z)
-    v = -np.cos(np.pi * x) * np.sin(np.pi * y) * np.cos(np.pi * z)
-    w = (np.sqrt(2.0 / 3.0) * np.cos(np.pi * x) * np.cos(np.pi * y) *
-            np.sin(np.pi * z))
-
-    ax.quiver(x, y, z, u, v, w, length=0.1, pivot='tail', normalize=True)
-
-
-@image_comparison(baseline_images=['poly3dcollection_closed'],
-                  remove_text=True)
+@mpl3d_image_comparison(['poly3dcollection_closed.png'])
 def test_poly3dcollection_closed():
     fig = plt.figure()
     ax = fig.gca(projection='3d')
@@ -454,8 +545,7 @@ def test_poly_collection_2d_to_3d_empty():
     assert poly.get_paths() == []
 
 
-@image_comparison(baseline_images=['poly3dcollection_alpha'],
-                  remove_text=True, extensions=['png'])
+@mpl3d_image_comparison(['poly3dcollection_alpha.png'])
 def test_poly3dcollection_alpha():
     fig = plt.figure()
     ax = fig.gca(projection='3d')
@@ -472,14 +562,12 @@ def test_poly3dcollection_alpha():
     ax.add_collection3d(c2)
 
 
-@image_comparison(baseline_images=['axes3d_labelpad'], extensions=['png'])
+@mpl3d_image_comparison(['axes3d_labelpad.png'], remove_text=False)
 def test_axes3d_labelpad():
-    from matplotlib import rcParams
-
     fig = plt.figure()
     ax = Axes3D(fig)
     # labelpad respects rcParams
-    assert ax.xaxis.labelpad == rcParams['axes.labelpad']
+    assert ax.xaxis.labelpad == mpl.rcParams['axes.labelpad']
     # labelpad can be set in set_label
     ax.set_xlabel('X LABEL', labelpad=10)
     assert ax.xaxis.labelpad == 10
@@ -494,7 +582,7 @@ def test_axes3d_labelpad():
         tick.set_pad(tick.get_pad() - i * 5)
 
 
-@image_comparison(baseline_images=['axes3d_cla'], extensions=['png'])
+@mpl3d_image_comparison(['axes3d_cla.png'], remove_text=False)
 def test_axes3d_cla():
     # fixed in pull request 4553
     fig = plt.figure()
@@ -503,6 +591,13 @@ def test_axes3d_cla():
     ax.cla()  # make sure the axis displayed is 3D (not 2D)
 
 
+@mpl3d_image_comparison(['axes3d_rotated.png'], remove_text=False)
+def test_axes3d_rotated():
+    fig = plt.figure()
+    ax = fig.add_subplot(1, 1, 1, projection='3d')
+    ax.view_init(90, 45)  # look down, rotated. Should be square
+
+
 def test_plotsurface_1d_raises():
     x = np.linspace(0.5, 10, num=100)
     y = np.linspace(0.5, 10, num=100)
@@ -558,8 +653,7 @@ def _test_proj_draw_axes(M, s=1, *args, **kwargs):
     return fig, ax
 
 
-@image_comparison(baseline_images=['proj3d_axes_cube'], extensions=['png'],
-                  remove_text=True, style='default')
+@mpl3d_image_comparison(['proj3d_axes_cube.png'])
 def test_proj_axes_cube():
     M = _test_proj_make_M()
 
@@ -581,8 +675,7 @@ def test_proj_axes_cube():
     ax.set_ylim(-0.2, 0.2)
 
 
-@image_comparison(baseline_images=['proj3d_axes_cube_ortho'],
-                  extensions=['png'], remove_text=True, style='default')
+@mpl3d_image_comparison(['proj3d_axes_cube_ortho.png'])
 def test_proj_axes_cube_ortho():
     E = np.array([200, 100, 100])
     R = np.array([0, 0, 0])
@@ -631,8 +724,7 @@ def test_world():
                                 [0, 0, 0, 1]])
 
 
-@image_comparison(baseline_images=['proj3d_lines_dists'], extensions=['png'],
-                  remove_text=True, style='default')
+@mpl3d_image_comparison(['proj3d_lines_dists.png'])
 def test_lines_dists():
     fig, ax = plt.subplots(figsize=(4, 6), subplot_kw=dict(aspect='equal'))
 
@@ -666,7 +758,7 @@ def test_autoscale():
     assert ax.get_w_lims() == (0, 1, -.1, 1.1, -.4, 2.4)
 
 
-@image_comparison(baseline_images=['axes3d_ortho'], style='default')
+@mpl3d_image_comparison(['axes3d_ortho.png'], remove_text=False)
 def test_axes3d_ortho():
     fig = plt.figure()
     ax = fig.gca(projection='3d')
@@ -690,12 +782,8 @@ def test_invalid_axes_limits(setter, side, value):
         getattr(obj, setter)(**limit)
 
 
-class TestVoxels(object):
-    @image_comparison(
-        baseline_images=['voxels-simple'],
-        extensions=['png'],
-        remove_text=True
-    )
+class TestVoxels:
+    @mpl3d_image_comparison(['voxels-simple.png'])
     def test_simple(self):
         fig, ax = plt.subplots(subplot_kw={"projection": "3d"})
 
@@ -703,12 +791,7 @@ class TestVoxels(object):
         voxels = (x == y) | (y == z)
         ax.voxels(voxels)
 
-    @image_comparison(
-        baseline_images=['voxels-edge-style'],
-        extensions=['png'],
-        remove_text=True,
-        style='default'
-    )
+    @mpl3d_image_comparison(['voxels-edge-style.png'])
     def test_edge_style(self):
         fig, ax = plt.subplots(subplot_kw={"projection": "3d"})
 
@@ -719,11 +802,7 @@ class TestVoxels(object):
         # change the edge color of one voxel
         v[max(v.keys())].set_edgecolor('C2')
 
-    @image_comparison(
-        baseline_images=['voxels-named-colors'],
-        extensions=['png'],
-        remove_text=True
-    )
+    @mpl3d_image_comparison(['voxels-named-colors.png'])
     def test_named_colors(self):
         """Test with colors set to a 3d object array of strings."""
         fig, ax = plt.subplots(subplot_kw={"projection": "3d"})
@@ -736,11 +815,7 @@ class TestVoxels(object):
         colors[(x + z) < 10] = 'cyan'
         ax.voxels(voxels, facecolors=colors)
 
-    @image_comparison(
-        baseline_images=['voxels-rgb-data'],
-        extensions=['png'],
-        remove_text=True
-    )
+    @mpl3d_image_comparison(['voxels-rgb-data.png'])
     def test_rgb_data(self):
         """Test with colors set to a 4d float array of rgb data."""
         fig, ax = plt.subplots(subplot_kw={"projection": "3d"})
@@ -753,11 +828,7 @@ class TestVoxels(object):
         colors[..., 2] = z / 9
         ax.voxels(voxels, facecolors=colors)
 
-    @image_comparison(
-        baseline_images=['voxels-alpha'],
-        extensions=['png'],
-        remove_text=True
-    )
+    @mpl3d_image_comparison(['voxels-alpha.png'])
     def test_alpha(self):
         fig, ax = plt.subplots(subplot_kw={"projection": "3d"})
 
@@ -775,11 +846,7 @@ class TestVoxels(object):
             assert voxels[coord], "faces returned for absent voxel"
             assert isinstance(poly, art3d.Poly3DCollection)
 
-    @image_comparison(
-        baseline_images=['voxels-xyz'],
-        extensions=['png'],
-        tol=0.01
-    )
+    @mpl3d_image_comparison(['voxels-xyz.png'], tol=0.01, remove_text=False)
     def test_xyz(self):
         fig, ax = plt.subplots(subplot_kw={"projection": "3d"})
 
@@ -826,13 +893,11 @@ class TestVoxels(object):
             ax.voxels(x, y, z, filled=filled, **kw)
 
         # duplicate argument
-        with pytest.raises(TypeError) as exc:
+        with pytest.raises(TypeError, match='voxels'):
             ax.voxels(x, y, z, filled, filled=filled)
-        exc.match(".*voxels.*")
         # missing arguments
-        with pytest.raises(TypeError) as exc:
+        with pytest.raises(TypeError, match='voxels'):
             ax.voxels(x, y)
-        exc.match(".*voxels.*")
         # x, y, z are positional only - this passes them on as attributes of
         # Poly3DCollection
         with pytest.raises(AttributeError):
@@ -864,7 +929,7 @@ def test_inverted(fig_test, fig_ref):
 
 
 def test_inverted_cla():
-    # Github PR #5450. Setting autoscale should reset
+    # GitHub PR #5450. Setting autoscale should reset
     # axes to be non-inverted.
     fig, ax = plt.subplots(subplot_kw={"projection": "3d"})
     # 1. test that a new axis is not inverted per default
@@ -883,56 +948,136 @@ def test_inverted_cla():
     assert not ax.zaxis_inverted()
 
 
-def test_art3d_deprecated():
+def test_ax3d_tickcolour():
+    fig = plt.figure()
+    ax = Axes3D(fig)
 
-    with pytest.warns(MatplotlibDeprecationWarning):
-        art3d.norm_angle(0.0)
+    ax.tick_params(axis='x', colors='red')
+    ax.tick_params(axis='y', colors='red')
+    ax.tick_params(axis='z', colors='red')
+    fig.canvas.draw()
 
-    with pytest.warns(MatplotlibDeprecationWarning):
-        art3d.norm_text_angle(0.0)
-
-    path = mpath.Path(np.empty((0, 2)))
-
-    with pytest.warns(MatplotlibDeprecationWarning):
-        art3d.path_to_3d_segment(path)
-
-    with pytest.warns(MatplotlibDeprecationWarning):
-        art3d.paths_to_3d_segments([path])
-
-    with pytest.warns(MatplotlibDeprecationWarning):
-        art3d.path_to_3d_segment_with_codes(path)
-
-    with pytest.warns(MatplotlibDeprecationWarning):
-        art3d.paths_to_3d_segments_with_codes([path])
-
-    with pytest.warns(MatplotlibDeprecationWarning):
-        art3d.get_colors([], 1)
-
-    with pytest.warns(MatplotlibDeprecationWarning):
-        art3d.zalpha([], [])
+    for tick in ax.xaxis.get_major_ticks():
+        assert tick.tick1line._color == 'red'
+    for tick in ax.yaxis.get_major_ticks():
+        assert tick.tick1line._color == 'red'
+    for tick in ax.zaxis.get_major_ticks():
+        assert tick.tick1line._color == 'red'
 
 
-def test_proj3d_deprecated():
-    with pytest.warns(MatplotlibDeprecationWarning):
-        proj3d.line2d([0, 1], [0, 1])
+@check_figures_equal(extensions=["png"])
+def test_ticklabel_format(fig_test, fig_ref):
+    axs = fig_test.subplots(4, 5, subplot_kw={"projection": "3d"})
+    for ax in axs.flat:
+        ax.set_xlim(1e7, 1e7 + 10)
+    for row, name in zip(axs, ["x", "y", "z", "both"]):
+        row[0].ticklabel_format(
+            axis=name, style="plain")
+        row[1].ticklabel_format(
+            axis=name, scilimits=(-2, 2))
+        row[2].ticklabel_format(
+            axis=name, useOffset=not mpl.rcParams["axes.formatter.useoffset"])
+        row[3].ticklabel_format(
+            axis=name, useLocale=not mpl.rcParams["axes.formatter.use_locale"])
+        row[4].ticklabel_format(
+            axis=name,
+            useMathText=not mpl.rcParams["axes.formatter.use_mathtext"])
 
-    with pytest.warns(MatplotlibDeprecationWarning):
-        proj3d.line2d_dist([0, 1, 3], [0, 1])
+    def get_formatters(ax, names):
+        return [getattr(ax, name).get_major_formatter() for name in names]
 
-    with pytest.warns(MatplotlibDeprecationWarning):
-        proj3d.mod([1, 1, 1])
+    axs = fig_ref.subplots(4, 5, subplot_kw={"projection": "3d"})
+    for ax in axs.flat:
+        ax.set_xlim(1e7, 1e7 + 10)
+    for row, names in zip(
+            axs, [["xaxis"], ["yaxis"], ["zaxis"], ["xaxis", "yaxis", "zaxis"]]
+    ):
+        for fmt in get_formatters(row[0], names):
+            fmt.set_scientific(False)
+        for fmt in get_formatters(row[1], names):
+            fmt.set_powerlimits((-2, 2))
+        for fmt in get_formatters(row[2], names):
+            fmt.set_useOffset(not mpl.rcParams["axes.formatter.useoffset"])
+        for fmt in get_formatters(row[3], names):
+            fmt.set_useLocale(not mpl.rcParams["axes.formatter.use_locale"])
+        for fmt in get_formatters(row[4], names):
+            fmt.set_useMathText(
+                not mpl.rcParams["axes.formatter.use_mathtext"])
 
-    vec = np.arange(4)
-    M = np.ones((4, 4))
 
-    with pytest.warns(MatplotlibDeprecationWarning):
-        proj3d.proj_transform_vec(vec, M)
+@check_figures_equal(extensions=["png"])
+def test_quiver3D_smoke(fig_test, fig_ref):
+    pivot = "middle"
+    # Make the grid
+    x, y, z = np.meshgrid(
+        np.arange(-0.8, 1, 0.2),
+        np.arange(-0.8, 1, 0.2),
+        np.arange(-0.8, 1, 0.8)
+    )
+    u = v = w = np.ones_like(x)
 
-    with pytest.warns(MatplotlibDeprecationWarning):
-        proj3d.proj_transform_vec_clip(vec, M)
+    for fig, length in zip((fig_ref, fig_test), (1, 1.0)):
+        ax = fig.gca(projection="3d")
+        ax.quiver(x, y, z, u, v, w, length=length, pivot=pivot)
 
-    with pytest.warns(MatplotlibDeprecationWarning):
-        proj3d.vec_pad_ones(np.ones(3), np.ones(3), np.ones(3))
 
-    with pytest.warns(MatplotlibDeprecationWarning):
-        proj3d.proj_trans_clip_points(np.ones((4, 3)), M)
+@image_comparison(["minor_ticks.png"], style="mpl20")
+def test_minor_ticks():
+    ax = plt.figure().add_subplot(projection="3d")
+    ax.set_xticks([0.25], minor=True)
+    ax.set_xticklabels(["quarter"], minor=True)
+    ax.set_yticks([0.33], minor=True)
+    ax.set_yticklabels(["third"], minor=True)
+    ax.set_zticks([0.50], minor=True)
+    ax.set_zticklabels(["half"], minor=True)
+
+
+@image_comparison(["equal_box_aspect.png"], style="mpl20")
+def test_equal_box_aspect():
+    from itertools import product, combinations
+
+    fig = plt.figure()
+    ax = fig.add_subplot(111, projection="3d")
+
+    # Make data
+    u = np.linspace(0, 2 * np.pi, 100)
+    v = np.linspace(0, np.pi, 100)
+    x = np.outer(np.cos(u), np.sin(v))
+    y = np.outer(np.sin(u), np.sin(v))
+    z = np.outer(np.ones_like(u), np.cos(v))
+
+    # Plot the surface
+    ax.plot_surface(x, y, z)
+
+    # draw cube
+    r = [-1, 1]
+    for s, e in combinations(np.array(list(product(r, r, r))), 2):
+        if np.sum(np.abs(s - e)) == r[1] - r[0]:
+            ax.plot3D(*zip(s, e), color="b")
+
+    # Make axes limits
+    xyzlim = np.column_stack(
+        [ax.get_xlim3d(), ax.get_ylim3d(), ax.get_zlim3d()]
+    )
+    XYZlim = [min(xyzlim[0]), max(xyzlim[1])]
+    ax.set_xlim3d(XYZlim)
+    ax.set_ylim3d(XYZlim)
+    ax.set_zlim3d(XYZlim)
+    ax.axis('off')
+    ax.set_box_aspect((1, 1, 1))
+
+
+def test_colorbar_pos():
+    num_plots = 2
+    fig, axs = plt.subplots(1, num_plots, figsize=(4, 5),
+                            constrained_layout=True,
+                            subplot_kw={'projection': '3d'})
+    for ax in axs:
+        p_tri = ax.plot_trisurf(np.random.randn(5), np.random.randn(5),
+                                np.random.randn(5))
+
+    cbar = plt.colorbar(p_tri, ax=axs, orientation='horizontal')
+
+    fig.canvas.draw()
+    # check that actually on the bottom
+    assert cbar.ax.get_position().extents[1] < 0.2
